from functools import partial
from email.utils import unquote
from email.mime.image import MIMEImage

from django.core.mail import make_msgid
from django.utils.module_loading import import_string
from django.conf import settings

import six


def _get_klass_from_config(config_variable, default):
    klass_path = getattr(settings, config_variable, default)
    if isinstance(klass_path, six.string_types):
        klass_path = import_string(klass_path)

    return klass_path


get_emailmessage_klass = partial(
    _get_klass_from_config,
    'TEMPLATED_EMAIL_EMAIL_MESSAGE_CLASS',
    'django.core.mail.EmailMessage'
)

get_emailmultialternatives_klass = partial(
    _get_klass_from_config,
    'TEMPLATED_EMAIL_EMAIL_MULTIALTERNATIVES_CLASS',
    'django.core.mail.EmailMultiAlternatives',
)


class InlineImage(object):

    def __init__(self, filename, content, subtype=None, domain=None):
        self.filename = filename
        self._content = content
        self.subtype = subtype
        self.domain = domain
        self._content_id = None

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content_id = None
        self._content = value

    def attach_to_message(self, message):
        if not self._content_id:
            self.generate_cid()
        image = MIMEImage(self.content, self.subtype)
        image.add_header('Content-Disposition', 'inline', filename=self.filename)
        image.add_header('Content-ID', self._content_id)
        message.attach(image)

    def generate_cid(self):
        self._content_id = make_msgid('img', self.domain)

    def __str__(self):
        if not self._content_id:
            self.generate_cid()
        return 'cid:' + unquote(self._content_id)



from django.conf.urls import url

from templated_email.views import ShowEmailView

app_name = 'templated_email'
urlpatterns = [
    url(r'^email/(?P<uuid>.*)/$', ShowEmailView.as_view(), name='show_email'),
]



import uuid
import hashlib
from io import BytesIO

from django.conf import settings
from django.core.mail import get_connection
from django.template import Context
from django.utils.translation import ugettext as _
from django.core.files.storage import default_storage

from templated_email.utils import (
    get_emailmessage_klass, get_emailmultialternatives_klass)
from templated_email.utils import InlineImage
from render_block import render_block_to_string, BlockNotFound


try:
    import html2text
except ImportError:
    html2text = None


class EmailRenderException(Exception):
    pass


class TemplateBackend(object):
    """
    Backend which uses Django's
    templates, and django's send_mail function.

    Heavily inspired by http://stackoverflow.com/questions/2809547/creating-email-templates-with-django

    Default / preferred behaviour works like so:
        templates named
            templated_email/<template_name>.email

        {% block subject %} declares the subject
        {% block plain %} declares text/plain
        {% block html %} declares text/html

    Legacy behaviour loads from:
        Subjects for email templates can be configured in one of two ways:

        * If you are using internationalisation, you can simply create entries for
          "<template_name> email subject" as a msgid in your PO file

        * Using a dictionary in settings.py, TEMPLATED_EMAIL_DJANGO_SUBJECTS,
          for e.g.:
          TEMPLATED_EMAIL_DJANGO_SUBJECTS = {
            'welcome':'Welcome to my website',
          }

    Subjects are templatable using the context, i.e. A subject
    that resolves to 'Welcome to my website, %(username)s', requires that
    the context passed in to the send() method contains 'username' as one
    of it's keys
    """

    def __init__(self, fail_silently=False,
                 template_prefix=None, template_suffix=None, **kwargs):
        self.template_prefix = template_prefix or getattr(settings, 'TEMPLATED_EMAIL_TEMPLATE_DIR', 'templated_email/')
        self.template_suffix = template_suffix or getattr(settings, 'TEMPLATED_EMAIL_FILE_EXTENSION', 'email')

    def attach_inline_images(self, message, context):
        for value in context.values():
            if isinstance(value, InlineImage):
                value.attach_to_message(message)

    def host_inline_image(self, inline_image):
        from templated_email.urls import app_name
        md5sum = hashlib.md5(inline_image.content).hexdigest()

        filename = inline_image.filename
        filename = app_name + '/' + md5sum + filename
        if not default_storage.exists(filename):
            filename = default_storage.save(filename,
                                            BytesIO(inline_image.content))
        return default_storage.url(filename)

    def _render_email(self, template_name, context,
                      template_dir=None, file_extension=None):
        response = {}
        errors = {}
        render_context = Context(context, autoescape=False)

        file_extension = file_extension or self.template_suffix
        if file_extension.startswith('.'):
            file_extension = file_extension[1:]
        template_extension = '.%s' % file_extension

        if isinstance(template_name, (tuple, list, )):
            prefixed_templates = template_name
        else:
            prefixed_templates = [template_name]

        full_template_names = []
        for one_prefixed_template in prefixed_templates:
            one_full_template_name = ''.join((template_dir or self.template_prefix, one_prefixed_template))
            if not one_full_template_name.endswith(template_extension):
                one_full_template_name += template_extension
            full_template_names.append(one_full_template_name)

        for part in ['subject', 'html', 'plain']:
            try:
                response[part] = render_block_to_string(full_template_names, part, render_context)
            except BlockNotFound as error:
                errors[part] = error

        if response == {}:
            raise EmailRenderException("Couldn't render email parts. Errors: %s"
                                       % errors)

        return response

    def get_email_message(self, template_name, context, from_email=None, to=None,
                          cc=None, bcc=None, headers=None,
                          template_prefix=None, template_suffix=None,
                          template_dir=None, file_extension=None,
                          attachments=None, create_link=False):

        if create_link:
            email_uuid = uuid.uuid4()
            link_context = dict(context)
            context['email_uuid'] = email_uuid.hex
            for key, value in context.items():
                if isinstance(value, InlineImage):
                    link_context[key] = self.host_inline_image(value)

        EmailMessage = get_emailmessage_klass()
        EmailMultiAlternatives = get_emailmultialternatives_klass()
        parts = self._render_email(template_name, context,
                                   template_prefix or template_dir,
                                   template_suffix or file_extension)
        plain_part = 'plain' in parts
        html_part = 'html' in parts

        if create_link and html_part:
            static_html_part = self._render_email(
                template_name, link_context,
                template_prefix or template_dir,
                template_suffix or file_extension)['html']
            from templated_email.models import SavedEmail
            SavedEmail.objects.create(content=static_html_part, uuid=email_uuid)

        if 'subject' in parts:
            subject = parts['subject']
        else:
            subject_dict = getattr(settings, 'TEMPLATED_EMAIL_DJANGO_SUBJECTS', {})
            if isinstance(template_name, (list, tuple)):
                for template in template_name:
                    if template in subject_dict:
                        subject_template = subject_dict[template]
                        break
                else:
                    subject_template = _('%s email subject' % template_name[0])
            else:
                subject_template = subject_dict.get(template_name,
                                                    _('%s email subject' % template_name))
            subject = subject_template % context
        subject = subject.strip('\n\r')  # strip newlines from subject

        if html_part and not plain_part and html2text and \
                getattr(settings, 'TEMPLATED_EMAIL_AUTO_PLAIN', True):
            parts['plain'] = html2text.html2text(parts['html'])
            plain_part = True

        if plain_part and not html_part:
            e = EmailMessage(
                subject,
                parts['plain'],
                from_email,
                to,
                cc=cc,
                bcc=bcc,
                headers=headers,
                attachments=attachments,
            )

        elif html_part and not plain_part:
            e = EmailMessage(
                subject,
                parts['html'],
                from_email,
                to,
                cc=cc,
                bcc=bcc,
                headers=headers,
                attachments=attachments,
            )
            e.content_subtype = 'html'

        elif plain_part and html_part:
            e = EmailMultiAlternatives(
                subject,
                parts['plain'],
                from_email,
                to,
                cc=cc,
                bcc=bcc,
                headers=headers,
                attachments=attachments,
            )
            e.attach_alternative(parts['html'], 'text/html')

        else:
            raise EmailRenderException("Please specify at a plain and/or html block.")

        self.attach_inline_images(e, context)
        return e

    def send(self, template_name, from_email, recipient_list, context,
             cc=None, bcc=None,
             fail_silently=False,
             headers=None,
             template_prefix=None, template_suffix=None,
             template_dir=None, file_extension=None,
             auth_user=None, auth_password=None,
             connection=None, attachments=None,
             create_link=False, **kwargs):

        connection = connection or get_connection(username=auth_user,
                                                  password=auth_password,
                                                  fail_silently=fail_silently)

        e = self.get_email_message(template_name, context, from_email=from_email,
                                   to=recipient_list, cc=cc, bcc=bcc, headers=headers,
                                   template_prefix=template_prefix,
                                   template_suffix=template_suffix,
                                   template_dir=template_dir,
                                   file_extension=file_extension,
                                   attachments=attachments,
                                   create_link=create_link)

        e.connection = connection

        try:
            e.send(fail_silently)
        except NameError:
            raise EmailRenderException("Couldn't render plain or html parts")

        return e.extra_headers.get('Message-Id', None)



from django.views.generic import DetailView

from templated_email.models import SavedEmail


class ShowEmailView(DetailView):
    model = SavedEmail
    template_name = 'templated_email/saved_email.html'
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'



from functools import partial

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from templated_email import send_templated_mail


class TemplatedEmailFormViewMixin(object):
    templated_email_template_name = None
    templated_email_send_on_success = True
    templated_email_send_on_failure = False
    templated_email_from_email = partial(getattr, settings, 'TEMPLATED_EMAIL_FROM_EMAIL', None)

    def templated_email_get_template_names(self, valid):
        if self.templated_email_template_name is None:
            raise ImproperlyConfigured(
                "TemplatedEmailFormViewMixin requires either a definition of "
                "'templated_email_template_name' or an implementation of 'templated_email_get_template_names()'")
        return [self.templated_email_template_name]

    def templated_email_get_context_data(self, **kwargs):
        return kwargs

    def templated_email_get_recipients(self, form):
        raise NotImplementedError('You must implement templated_email_get_recipients method')

    def templated_email_get_send_email_kwargs(self, valid, form):
        if valid:
            context = self.templated_email_get_context_data(form_data=form.data)
        else:
            context = self.templated_email_get_context_data(form_errors=form.errors)
        try:
            from_email = self.templated_email_from_email()
        except TypeError:
            from_email = self.templated_email_from_email
        return {
            'template_name': self.templated_email_get_template_names(valid=valid),
            'from_email': from_email,
            'recipient_list': self.templated_email_get_recipients(form),
            'context': context
        }

    def templated_email_send_templated_mail(self, *args, **kwargs):
        return send_templated_mail(*args, **kwargs)

    def form_valid(self, form):
        response = super(TemplatedEmailFormViewMixin, self).form_valid(form)
        if self.templated_email_send_on_success:
            self.templated_email_send_templated_mail(
                **self.templated_email_get_send_email_kwargs(valid=True, form=form))
        return response

    def form_invalid(self, form):
        response = super(TemplatedEmailFormViewMixin, self).form_invalid(form)
        if self.templated_email_send_on_failure:
            self.templated_email_send_templated_mail(
                **self.templated_email_get_send_email_kwargs(valid=False, form=form))
        return response



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-05 17:09
from __future__ import unicode_literals

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SavedEmail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4)),
                ('content', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



from uuid import uuid4

from django.db import models


class SavedEmail(models.Model):
    uuid = models.UUIDField(default=uuid4)
    content = models.TextField()
    created = models.DateTimeField(auto_now_add=True)



import os
import sys
from setuptools import setup, find_packages

DESCRIPTION = "A Django oriented templated / transaction email abstraction"
VERSION = '2.1'
LONG_DESCRIPTION = None
try:
    LONG_DESCRIPTION = open('README.rst').read()
except:
    pass

requirements = [
    'django-render-block>=0.5',
    'six>=1',
]

# python setup.py register
if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    args = {'version': VERSION}
    print("You probably want to also tag the version now:")
    print("  git tag -a %(version)s -m 'version %(version)s'" % args)
    print("  git push --tags")
    sys.exit()

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Framework :: Django',
]

setup(
    name='django-templated-email',
    version=VERSION,
    packages=find_packages(exclude=("tests", "tests.*")),
    include_package_data=True,
    author='Bradley Whittington',
    author_email='radbrad182@gmail.com',
    url='http://github.com/vintasoftware/django-templated-email/',
    license='MIT',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    platforms=['any'],
    classifiers=CLASSIFIERS,
    install_requires=requirements,
)
